import sys
import subprocess
import time
import requests
import datetime
import platform
import socket
from PyQt5.QtWidgets import (QApplication, QWidget, QVBoxLayout, QHBoxLayout,
                             QPushButton, QLabel, QListWidget, QListWidgetItem,
                             QSizePolicy, QSpacerItem, QMenu, QFileDialog, 
                             QMessageBox, QProgressBar)
from PyQt5.QtCore import QThread, pyqtSignal, Qt, QSize, QPoint, QTimer
from PyQt5.QtGui import QFont, QCursor, QIcon, QColor
from PyQt5.QtNetwork import QNetworkConfigurationManager


class DownloadSpeedTestThread(QThread):
    update_speed = pyqtSignal(float)
    finished = pyqtSignal()
    error_occurred = pyqtSignal(str)
    progress_updated = pyqtSignal(int)
    
    def __init__(self):
        super().__init__()
        self._running = True
        self.max_speed = 0
        self.test_urls = [
            'https://hnd-jp-ping.vultr.com/vultr.com.100MB.bin',
            'https://wirelesscdn-download.xuexi.cn/publish/xuexi_android/latest/xuexi_android_10002068.apk',
            'https://speed.cloudflare.com/__down?during=download&bytes=104857600'
        ]
        self.timeout = 15  # seconds
        self.chunk_size = 1024 * 1024  # 1MB chunk

    def run(self):
        for url in self.test_urls:
            try:
                start_time = time.time()
                with requests.get(url, stream=True, timeout=(10, self.timeout)) as response:
                    response.raise_for_status()
                    total_size = int(response.headers.get('content-length', 0))
                    downloaded = 0
                    
                    for data in response.iter_content(chunk_size=self.chunk_size):
                        if not self._running:
                            break
                            
                        chunk_len = len(data)
                        downloaded += chunk_len
                        elapsed = time.time() - start_time
                        
                        # Calculate progress percentage if total size is known
                        if total_size > 0:
                            progress = int((downloaded / total_size) * 100)
                            self.progress_updated.emit(progress)
                        
                        if elapsed > 0.1:  # Avoid initial spike
                            speed = downloaded / elapsed
                            current_speed = speed * 8 / (1024 ** 2)  # Convert to Mbps
                            if current_speed > self.max_speed:
                                self.max_speed = current_speed
                            self.update_speed.emit(current_speed)
                            
                    if downloaded > 0:
                        self.finished.emit()
                        return
                        
            except requests.exceptions.RequestException as e:
                continue  # Try next URL
            except Exception as e:
                self.error_occurred.emit(f"Unexpected error: {str(e)}")
                continue
                
        # All URLs failed
        self.error_occurred.emit("无法连接到测试服务器")
        self.update_speed.emit(0.0)
        self.finished.emit()

    def stop(self):
        self._running = False


class PingTestThread(QThread):
    ping_result = pyqtSignal(str, str, float)  # domain, result, avg_time

    def __init__(self, domain):
        super().__init__()
        self.domain = domain
        self.ping_count = 4
        self.timeout = 4  # seconds

    def run(self):
        try:
            # Determine ping command based on OS
            if platform.system().lower() == "windows":
                ping_cmd = ['ping', '-n', str(self.ping_count), '-w', str(self.timeout*1000), self.domain]
            else:
                ping_cmd = ['ping', '-c', str(self.ping_count), '-W', str(self.timeout), self.domain]
                
            output = subprocess.check_output(
                ping_cmd,
                shell=True,
                stderr=subprocess.STDOUT,
                text=True,
                encoding='gbk'
            )
            
            times = []
            for line in output.split('\n'):
                if 'time=' in line or '时间=' in line:
                    try:
                        # Handle both Chinese and English formats
                        time_str = line.split('时间=')[1].split('ms')[0].strip() if '时间=' in line else line.split('time=')[1].split('ms')[0].strip()
                        times.append(float(time_str))
                    except (IndexError, ValueError):
                        continue
            
            if times:
                avg_time = sum(times)/len(times)
                self.ping_result.emit(self.domain, f"{avg_time:.1f} ms", avg_time)
            else:
                self.ping_result.emit(self.domain, "请求超时", float('inf'))
                
        except subprocess.CalledProcessError as e:
            error_msg = e.output.split('\n')[0] if e.output else str(e)
            self.ping_result.emit(self.domain, f"测试失败: {error_msg[:30]}", float('inf'))
        except Exception as e:
            self.ping_result.emit(self.domain, f"错误: {str(e)[:20]}", float('inf'))


class NetworkInfoThread(QThread):
    network_info_ready = pyqtSignal(dict)
    
    def run(self):
        info = {}
        try:
            # Get local IP address
            hostname = socket.gethostname()
            local_ip = socket.gethostbyname(hostname)
            info['local_ip'] = local_ip
            
            # Get public IP
            try:
                public_ip = requests.get('https://api.ipify.cn/?format=json', timeout=5).text
                info['public_ip'] = public_ip
            except:
                info['public_ip'] = "无法获取"
                
            # Get DNS info
            try:
                dns_info = socket.getaddrinfo('www.google.com', 80)
                info['dns_working'] = bool(dns_info)
            except:
                info['dns_working'] = False
                
        except Exception as e:
            info['error'] = str(e)
            
        self.network_info_ready.emit(info)


class SpeedTestApp(QWidget):
    def __init__(self):
        super().__init__()
        self.domains = [
            'www.baidu.com',
            'player.bilibili.com',
            'bing.com',
            'cloud.tencent.com',
            'github.com',
            'www.aliyun.com',
            '119.29.29.29',
            'cloudflare.com',
            '1.1.1.1'
        ]
        self.active_threads = []
        self.download_speed_history = []
        self.current_speed = 0
        self.max_speed = 0
        self.ping_results = {}
        self.network_info = {}
        self.test_start_time = None
        self.initUI()
        self.check_network_status()

    def initUI(self):
        self.setWindowTitle('Eric Speed')
        self.setWindowIcon(QIcon('speed.ico'))  # Add an icon if available
        self.setMinimumSize(600, 450)
        self.resize(800, 600)

        main_layout = QVBoxLayout()
        main_layout.setContentsMargins(20, 20, 20, 20)
        main_layout.setSpacing(15)

        # Network status bar
        self.network_status_bar = QHBoxLayout()
        self.network_status_label = QLabel('网络状态: 检测中...')
        self.network_status_label.setFont(QFont('Microsoft YaHei', 10))
        self.network_status_bar.addWidget(self.network_status_label)
        self.network_status_bar.addStretch()
        
        self.ip_label = QLabel('IP: -')
        self.ip_label.setFont(QFont('Microsoft YaHei', 10))
        self.ip_label.setToolTip("点击查看网络详情")
        self.ip_label.mousePressEvent = self.show_network_details
        self.network_status_bar.addWidget(self.ip_label)
        main_layout.addLayout(self.network_status_bar)

        # Top info area
        info_layout = QVBoxLayout()
        info_layout.setSpacing(15)
        
        # Status label
        self.status_label = QLabel('准备就绪')
        self.status_label.setFont(QFont('Microsoft YaHei', 12, QFont.Bold))
        info_layout.addWidget(self.status_label)

        # Download speed display
        speed_layout = QHBoxLayout()
        self.download_label = QLabel('下载速度: 0.00 Mbps')
        self.download_label.setFont(QFont('Microsoft YaHei', 14, QFont.Bold))
        speed_layout.addWidget(self.download_label)
        
        self.max_speed_label = QLabel('峰值速度: 0.00 Mbps')
        self.max_speed_label.setFont(QFont('Microsoft YaHei', 12))
        speed_layout.addWidget(self.max_speed_label)
        
        info_layout.addLayout(speed_layout)

        # Progress bar
        self.progress_bar = QProgressBar()
        self.progress_bar.setRange(0, 100)
        self.progress_bar.setTextVisible(True)
        self.progress_bar.setFormat("测试进度: %p%")
        info_layout.addWidget(self.progress_bar)

        # Control buttons
        button_layout = QHBoxLayout()
        
        self.start_btn = QPushButton('开始测试')
        self.start_btn.setFixedHeight(40)
        self.start_btn.setFont(QFont('Microsoft YaHei', 11))
        self.start_btn.clicked.connect(self.start_test)
        button_layout.addWidget(self.start_btn)
        
        self.stop_btn = QPushButton('停止测试')
        self.stop_btn.setFixedHeight(40)
        self.stop_btn.setFont(QFont('Microsoft YaHei', 11))
        self.stop_btn.clicked.connect(self.stop_test)
        self.stop_btn.setEnabled(False)
        button_layout.addWidget(self.stop_btn)
        
        self.export_btn = QPushButton('导出报告')
        self.export_btn.setFixedHeight(40)
        self.export_btn.setFont(QFont('Microsoft YaHei', 11))
        self.export_btn.clicked.connect(self.export_report)
        self.export_btn.setEnabled(False)
        button_layout.addWidget(self.export_btn)
        
        button_layout.addStretch()
        info_layout.addLayout(button_layout)

        main_layout.addLayout(info_layout)

        # Ping results list
        self.ping_list = QListWidget()
        self.ping_list.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.ping_list.setFont(QFont('Microsoft YaHei', 11))
        self.ping_list.setContextMenuPolicy(Qt.CustomContextMenu)
        self.ping_list.customContextMenuRequested.connect(self.show_context_menu)
        main_layout.addWidget(self.ping_list)

        # Bottom status bar
        self.status_bar = QLabel()
        self.status_bar.setFont(QFont('Microsoft YaHei', 9))
        main_layout.addWidget(self.status_bar)

        self.setLayout(main_layout)
        self.apply_stylesheet()

    def apply_stylesheet(self):
        self.setStyleSheet("""
            QWidget {
                background-color: #f5f5f5;
            }
            QLabel { 
                font-weight: bold; 
                padding: 5px;
            }
            QListWidget { 
                background-color: #ffffff; 
                border: 1px solid #ddd;
                border-radius: 4px;
                padding: 5px;
            }
            QPushButton { 
                background-color: #4CAF50;
                color: white;
                padding: 8px 16px;
                border-radius: 4px;
                min-width: 100px;
                border: none;
            }
            QPushButton:hover {
                background-color: #45a049;
            }
            QPushButton:disabled { 
                background-color: #cccccc; 
            }
            QPushButton#stop_btn {
                background-color: #f44336;
            }
            QPushButton#stop_btn:hover {
                background-color: #d32f2f;
            }
            QPushButton#export_btn {
                background-color: #2196F3;
            }
            QPushButton#export_btn:hover {
                background-color: #0b7dda;
            }
            QProgressBar {
                border: 1px solid #ddd;
                border-radius: 3px;
                text-align: center;
            }
            QProgressBar::chunk {
                background-color: #4CAF50;
                width: 10px;
            }
        """)
        
        self.stop_btn.setObjectName("stop_btn")
        self.export_btn.setObjectName("export_btn")

    def check_network_status(self):
        netman = QNetworkConfigurationManager()
        if netman.isOnline():
            self.network_status_label.setText("网络状态: 已连接")
            self.network_status_label.setStyleSheet("color: green;")
            
            # Get network info in background
            self.network_info_thread = NetworkInfoThread()
            self.network_info_thread.network_info_ready.connect(self.update_network_info)
            self.network_info_thread.start()
        else:
            self.network_status_label.setText("网络状态: 未连接")
            self.network_status_label.setStyleSheet("color: red;")
            self.ip_label.setText("IP: 无网络连接")

    def update_network_info(self, info):
        self.network_info = info
        if 'local_ip' in info:
            ip_text = f"本地IP: {info['local_ip']}"
            if 'public_ip' in info and info['public_ip'] != "无法获取":
                ip_text += f" | 公网IP: {info['public_ip']}"
            self.ip_label.setText(ip_text)

    def show_network_details(self, event):
        msg = QMessageBox()
        msg.setWindowTitle("网络详情")
        msg.setIcon(QMessageBox.Information)
        
        text = "<b>网络连接信息</b><br><br>"
        if 'local_ip' in self.network_info:
            text += f"<b>本地IP:</b> {self.network_info['local_ip']}<br>"
        if 'public_ip' in self.network_info:
            text += f"<b>公网IP:</b> {self.network_info['public_ip']}<br>"
        text += f"<b>DNS状态:</b> {'正常' if self.network_info.get('dns_working', False) else '异常'}<br>"
        
        if 'error' in self.network_info:
            text += f"<br><b>错误信息:</b> {self.network_info['error']}"
            
        msg.setText(text)
        msg.exec_()

    def start_test(self):
        self.cleanup_threads()
        self.start_btn.setEnabled(False)
        self.stop_btn.setEnabled(True)
        self.export_btn.setEnabled(False)
        self.status_label.setText('测试进行中...')
        self.status_label.setStyleSheet("color: black;")
        self.download_label.setText('下载速度: 测试中...')
        self.download_label.setStyleSheet("color: black;")
        self.max_speed_label.setText('峰值速度: 测试中...')
        self.max_speed_label.setStyleSheet("color: black;")
        self.progress_bar.setValue(0)
        self.ping_list.clear()
        self.download_speed_history = []
        self.current_speed = 0
        self.max_speed = 0
        self.ping_results = {}
        self.test_start_time = datetime.datetime.now()
        
        # Update status bar
        self.status_bar.setText(f"测试开始时间: {self.test_start_time.strftime('%Y-%m-%d %H:%M:%S')}")

        # Start download test
        self.download_thread = DownloadSpeedTestThread()
        self.download_thread.update_speed.connect(self.update_download_speed)
        self.download_thread.finished.connect(self.on_download_finished)
        self.download_thread.error_occurred.connect(self.show_error)
        self.download_thread.progress_updated.connect(self.progress_bar.setValue)
        self.active_threads.append(self.download_thread)
        self.download_thread.start()

        # Start ping tests
        for domain in self.domains:
            item = QListWidgetItem(f"{domain}: 测试中...")
            item.setData(Qt.UserRole, domain)  # Store domain for reference
            item.setForeground(Qt.darkGray)
            item.setSizeHint(QSize(0, 35))
            self.ping_list.addItem(item)
            
            ping_thread = PingTestThread(domain)
            ping_thread.ping_result.connect(self.update_ping_item)
            self.active_threads.append(ping_thread)
            ping_thread.start()

    def stop_test(self):
        self.status_label.setText("测试已停止")
        self.status_label.setStyleSheet("color: orange;")
        self.cleanup_threads()
        self.stop_btn.setEnabled(False)
        self.start_btn.setEnabled(True)
        self.export_btn.setEnabled(True)

    def cleanup_threads(self):
        for thread in self.active_threads:
            if thread.isRunning():
                if hasattr(thread, 'stop'):
                    thread.stop()
                thread.quit()
                thread.wait(1000)
        self.active_threads = []

    def update_download_speed(self, speed):
        self.current_speed = speed
        self.download_speed_history.append(speed)
        
        # Update max speed if needed
        if speed > self.max_speed:
            self.max_speed = speed
            self.max_speed_label.setText(f'峰值速度: {self.max_speed:.2f} Mbps')
            self.max_speed_label.setStyleSheet("color: darkgreen;")
            
        # Update speed display with color coding
        self.download_label.setText(f'下载速度: {speed:.2f} Mbps')
        if speed < 1:
            self.download_label.setStyleSheet("color: red;")
        elif speed < 10:
            self.download_label.setStyleSheet("color: orange;")
        else:
            self.download_label.setStyleSheet("color: green;")

    def update_ping_item(self, domain, result, avg_time):
        items = self.ping_list.findItems(domain, Qt.MatchContains)
        if items:
            item = items[0]
            item.setText(f"{domain}: {result}")
            self.ping_results[domain] = avg_time
            
            # Color code based on latency
            if "ms" in result:
                try:
                    latency = float(result.split(' ')[0])
                    if latency > 300:
                        item.setForeground(Qt.red)
                    elif latency > 200:
                        item.setForeground(QColor(255, 165, 0))  # Orange
                    elif latency > 100:
                        item.setForeground(QColor(255, 215, 0))  # Gold
                    else:
                        item.setForeground(Qt.darkGreen)
                except ValueError:
                    item.setForeground(Qt.blue)
            elif "失败" in result or "错误" in result:
                item.setForeground(Qt.red)

    def on_download_finished(self):
        self.status_label.setText("下载测试完成")
        self.status_label.setStyleSheet("color: green;")
        self.stop_btn.setEnabled(False)
        self.start_btn.setEnabled(True)
        self.export_btn.setEnabled(True)
        
        # Update status bar with test duration
        if self.test_start_time:
            duration = datetime.datetime.now() - self.test_start_time
            self.status_bar.setText(
                f"测试完成 | 持续时间: {duration.total_seconds():.1f}秒 | " +
                f"峰值速度: {self.max_speed:.2f} Mbps"
            )

    def show_error(self, error_msg):
        self.status_label.setText(f"错误: {error_msg}")
        self.status_label.setStyleSheet("color: red;")
        QMessageBox.warning(self, "测试错误", error_msg)
        self.stop_btn.setEnabled(False)
        self.start_btn.setEnabled(True)

    def show_context_menu(self, pos):
        item = self.ping_list.itemAt(pos)
        if not item:
            return
            
        menu = QMenu()
        copy_action = menu.addAction("复制结果")
        retry_action = menu.addAction("重新测试")
        action = menu.exec_(self.ping_list.mapToGlobal(pos))
        
        if action == copy_action:
            clipboard = QApplication.clipboard()
            clipboard.setText(item.text())
        elif action == retry_action:
            domain = item.data(Qt.UserRole)
            self.retry_ping_test(domain, item)

    def retry_ping_test(self, domain, item):
        item.setText(f"{domain}: 重新测试中...")
        item.setForeground(Qt.darkGray)
        
        ping_thread = PingTestThread(domain)
        ping_thread.ping_result.connect(self.update_ping_item)
        self.active_threads.append(ping_thread)
        ping_thread.start()

    def export_report(self):
        options = QFileDialog.Options()
        fileName, _ = QFileDialog.getSaveFileName(
            self, "保存测试报告", 
            f"网络测速报告_{datetime.datetime.now().strftime('%Y%m%d_%H%M%S')}.txt", 
            "文本文件 (*.txt);;HTML 文件 (*.html)", 
            options=options
        )
        
        if not fileName:
            return
            
        try:
            if fileName.endswith('.html'):
                self.export_html_report(fileName)
            else:
                self.export_text_report(fileName)
                
            self.status_label.setText(f"报告已保存到: {fileName}")
            self.status_label.setStyleSheet("color: darkgreen;")
        except Exception as e:
            self.status_label.setText(f"保存报告失败: {str(e)}")
            self.status_label.setStyleSheet("color: red;")
            QMessageBox.critical(self, "导出错误", f"无法保存报告:\n{str(e)}")

    def export_text_report(self, filename):
        with open(filename, 'w', encoding='utf-8') as f:
            # Report header
            f.write("=== 网络测速报告 ===\n\n")
            f.write(f"生成时间: {datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')}\n")
            f.write("来自Eric Speed \n\n")

            # Network info
            f.write("\n=== 网络信息 ===\n")
            if 'local_ip' in self.network_info:
                f.write(f"本地IP: {self.network_info['local_ip']}\n")
            if 'public_ip' in self.network_info:
                f.write(f"公网IP: {self.network_info['public_ip']}\n")
            f.write(f"DNS状态: {'正常' if self.network_info.get('dns_working', False) else '异常'}\n")
            
            # Download speed results
            f.write("\n=== 下载速度测试 ===\n")
            if self.download_speed_history:
                avg_speed = sum(self.download_speed_history)/len(self.download_speed_history)
                f.write(f"当前下载速度: {self.current_speed:.2f} Mbps\n")
                f.write(f"峰值下载速度: {self.max_speed:.2f} Mbps\n")
                f.write(f"平均下载速度: {avg_speed:.2f} Mbps\n")
                
                # Speed evaluation
                speed_eval = self.get_speed_evaluation(self.max_speed)
                f.write(f"\n评分等级: {speed_eval[0]}\n")
                f.write(f"性能描述: {speed_eval[1]}\n")
                f.write("改进建议:\n")
                for suggestion in speed_eval[2]:
                    f.write(f"  • {suggestion}\n")
            else:
                f.write("下载速度测试未完成或无有效数据\n")
            
            # Ping results
            f.write("\n=== 网络延迟测试 ===\n")
            valid_pings = [t for t in self.ping_results.values() if t != float('inf')]
            for i in range(self.ping_list.count()):
                item = self.ping_list.item(i)
                f.write(f"{item.text()}\n")
            
            if valid_pings:
                avg_ping = sum(valid_pings)/len(valid_pings)
                min_ping = min(valid_pings)
                max_ping = max(valid_pings)
                
                f.write(f"\n平均延迟: {avg_ping:.1f} ms\n")
                f.write(f"最低延迟: {min_ping:.1f} ms\n")
                f.write(f"最高延迟: {max_ping:.1f} ms\n")
                
                # Ping evaluation
                ping_eval = self.get_ping_evaluation(f"{avg_ping:.1f} ms")
                f.write(f"\n延迟评分: {ping_eval[0]}\n")
                f.write(f"质量描述: {ping_eval[1]}\n")
                f.write("优化建议:\n")
                for suggestion in ping_eval[2]:
                    f.write(f"  • {suggestion}\n")
            
            # Overall evaluation
            f.write("\n=== 总体网络评估 ===\n")
            if self.download_speed_history and valid_pings:
                speed_eval = self.get_speed_evaluation(self.max_speed)
                ping_eval = self.get_ping_evaluation(f"{avg_ping:.1f} ms")
                overall_eval = self.get_overall_evaluation(speed_eval, ping_eval)
                
                f.write(f"综合评级: {overall_eval[0]}\n")
                f.write(f"综合描述: {overall_eval[1]}\n")
                f.write("优化方案:\n")
                for solution in overall_eval[2]:
                    f.write(f"  * {solution}\n")
            else:
                f.write("无法完成综合评估 - 测试数据不完整\n")
            
            # Test summary
            f.write("\n=== 测试摘要 ===\n")
            if self.test_start_time:
                duration = datetime.datetime.now() - self.test_start_time
                f.write(f"测试持续时间: {duration.total_seconds():.1f} 秒\n")
            f.write(f"测试节点数量: {len(self.domains)}个\n")
            f.write(f"有效延迟测试: {len(valid_pings)}次\n")
            f.write("(测速可能会因为多方面原因导致不准 可以看看这个 https://zhuanlan.zhihu.com/p/159277005)\n\n")

    def export_html_report(self, filename):
        with open(filename, 'w', encoding='utf-8') as f:
            # HTML header
            f.write("""<!DOCTYPE html>
<html>
<head>
    <meta charset="UTF-8">
    <title>网络测速报告</title>
    <style>
        body { font-family: Arial, sans-serif; margin: 20px; }
        h1, h2 { color: #333; }
        .header { background-color: #f0f0f0; padding: 15px; border-radius: 5px; }
        .section { margin-bottom: 20px; border-bottom: 1px solid #ddd; padding-bottom: 15px; }
        .speed-good { color: green; }
        .speed-medium { color: orange; }
        .speed-bad { color: red; }
        .ping-good { color: green; }
        .ping-medium { color: orange; }
        .ping-bad { color: red; }
        table { border-collapse: collapse; width: 100%; margin-bottom: 15px; }
        th, td { border: 1px solid #ddd; padding: 8px; text-align: left; }
        th { background-color: #f2f2f2; }
        .suggestions { margin-left: 20px; }
    </style>
</head>
<body>
""")
            
            # Report header
            f.write(f"""
<div class="header">
    <h1>网络测速报告</h1>
    <p>生成时间: {datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')}</p>
</div>
""")
            
            # Network info
            f.write("""
<div class="section">
    <h2>网络信息</h2>
    <table>
        <tr>
            <th>项目</th>
            <th>值</th>
        </tr>
""")
            
            if 'local_ip' in self.network_info:
                f.write(f"""
        <tr>
            <td>本地IP地址</td>
            <td>{self.network_info['local_ip']}</td>
        </tr>
""")
            
            if 'public_ip' in self.network_info:
                f.write(f"""
        <tr>
            <td>公网IP地址</td>
            <td>{self.network_info['public_ip']}</td>
        </tr>
""")
            
            f.write(f"""
        <tr>
            <td>DNS状态</td>
            <td>{'<span style="color:green">正常</span>' if self.network_info.get('dns_working', False) else '<span style="color:red">异常</span>'}</td>
        </tr>
    </table>
</div>
""")
            
            # Download speed results
            if self.download_speed_history:
                avg_speed = sum(self.download_speed_history)/len(self.download_speed_history)
                speed_eval = self.get_speed_evaluation(self.max_speed)
                speed_class = "speed-good" if self.max_speed >= 10 else "speed-medium" if self.max_speed >= 5 else "speed-bad"
                
                f.write(f"""
<div class="section">
    <h2>下载速度测试</h2>
    <table>
        <tr>
            <th>指标</th>
            <th>值</th>
        </tr>
        <tr>
            <td>当前下载速度</td>
            <td class="{speed_class}">{self.current_speed:.2f} Mbps</td>
        </tr>
        <tr>
            <td>峰值下载速度</td>
            <td class="{speed_class}">{self.max_speed:.2f} Mbps</td>
        </tr>
        <tr>
            <td>平均下载速度</td>
            <td class="{speed_class}">{avg_speed:.2f} Mbps</td>
        </tr>
        <tr>
            <td>评分等级</td>
            <td><strong>{speed_eval[0]}</strong></td>
        </tr>
        <tr>
            <td>性能描述</td>
            <td>{speed_eval[1]}</td>
        </tr>
    </table>
    
    <h3>改进建议</h3>
    <ul class="suggestions">
""")
                
                for suggestion in speed_eval[2]:
                    f.write(f"        <li>{suggestion}</li>\n")
                
                f.write("""
    </ul>
</div>
""")
            
            # Ping results
            valid_pings = [t for t in self.ping_results.values() if t != float('inf')]
            if valid_pings:
                avg_ping = sum(valid_pings)/len(valid_pings)
                min_ping = min(valid_pings)
                max_ping = max(valid_pings)
                ping_eval = self.get_ping_evaluation(f"{avg_ping:.1f} ms")
                ping_class = "ping-good" if avg_ping <= 100 else "ping-medium" if avg_ping <= 200 else "ping-bad"
                
                f.write("""
<div class="section">
    <h2>网络延迟测试</h2>
    <table>
        <tr>
            <th>服务器</th>
            <th>延迟</th>
        </tr>
""")
                
                for i in range(self.ping_list.count()):
                    item = self.ping_list.item(i)
                    text = item.text()
                    if "ms" in text:
                        latency = float(text.split(":")[1].split("ms")[0].strip())
                        row_class = "ping-good" if latency <= 100 else "ping-medium" if latency <= 200 else "ping-bad"
                        f.write(f"""
        <tr>
            <td>{text.split(':')[0]}</td>
            <td class="{row_class}">{text.split(':')[1].strip()}</td>
        </tr>
""")
                    else:
                        f.write(f"""
        <tr>
            <td>{text.split(':')[0]}</td>
            <td class="ping-bad">{text.split(':')[1].strip()}</td>
        </tr>
""")
                
                f.write(f"""
    </table>
    
    <h3>延迟统计</h3>
    <table>
        <tr>
            <th>指标</th>
            <th>值</th>
        </tr>
        <tr>
            <td>平均延迟</td>
            <td class="{ping_class}">{avg_ping:.1f} ms</td>
        </tr>
        <tr>
            <td>最低延迟</td>
            <td class="ping-good">{min_ping:.1f} ms</td>
        </tr>
        <tr>
            <td>最高延迟</td>
            <td class="{'ping-medium' if max_ping <= 300 else 'ping-bad'}">{max_ping:.1f} ms</td>
        </tr>
        <tr>
            <td>评分等级</td>
            <td><strong>{ping_eval[0]}</strong></td>
        </tr>
        <tr>
            <td>质量描述</td>
            <td>{ping_eval[1]}</td>
        </tr>
    </table>
    
    <h3>优化建议</h3>
    <ul class="suggestions">
""")
                
                for suggestion in ping_eval[2]:
                    f.write(f"        <li>{suggestion}</li>\n")
                
                f.write("""
    </ul>
</div>
""")
            
            # Overall evaluation
            if self.download_speed_history and valid_pings:
                speed_eval = self.get_speed_evaluation(self.max_speed)
                ping_eval = self.get_ping_evaluation(f"{avg_ping:.1f} ms")
                overall_eval = self.get_overall_evaluation(speed_eval, ping_eval)
                
                f.write(f"""
<div class="section">
    <h2>总体网络评估</h2>
    <table>
        <tr>
            <th>指标</th>
            <th>值</th>
        </tr>
        <tr>
            <td>综合评级</td>
            <td><strong>{overall_eval[0]}</strong></td>
        </tr>
        <tr>
            <td>综合描述</td>
            <td>{overall_eval[1]}</td>
        </tr>
    </table>
    
    <h3>优化方案</h3>
    <ul class="suggestions">
""")
                
                for solution in overall_eval[2]:
                    f.write(f"        <li>{solution}</li>\n")
                
                f.write("""
    </ul>
</div>
""")
            
            # Test summary
            f.write("""
<div class="section">
    <h2>测试摘要</h2>
    <table>
""")
            
            if self.test_start_time:
                duration = datetime.datetime.now() - self.test_start_time
                f.write(f"""
        <tr>
            <td>测试持续时间</td>
            <td>{duration.total_seconds():.1f} 秒</td>
        </tr>
""")
            
            f.write(f"""
        <tr>
            <td>测试节点数量</td>
            <td>{len(self.domains)}个</td>
        </tr>
        <tr>
            <td>有效延迟测试</td>
            <td>{len(valid_pings)}次</td>
        </tr>
    </table>
</div>
""")
            
            # HTML footer
            f.write("""
</body>
</html>
""")

    def get_speed_evaluation(self, speed):
        if speed == 0:
            return ("极差", "网络连接完全不可用", [
                "检查网络连接是否正常",
                "确认路由器/调制解调器工作正常",
                "尝试重启网络设备",
                "联系您的网络服务提供商"
            ])
        elif speed < 1:
            return ("非常差", "仅能支持最基本的文本浏览", [
                "检查是否有后台程序占用带宽",
                "尝试关闭其他设备连接",
                "考虑升级网络套餐",
                "检查线路是否有物理损坏"
            ])
        elif speed < 5:
            return ("较差", "可以浏览网页但视频加载缓慢", [
                "优化路由器位置",
                "检查是否有网络限速",
                "考虑使用有线连接代替WiFi",
                "关闭不必要的网络应用"
            ])
        elif speed < 10:
            return ("一般", "适合标清视频流媒体", [
                "确保使用5GHz WiFi频段(如适用)",
                "检查网络设备是否支持当前速度",
                "定期重启网络设备",
                "考虑升级到更高带宽套餐"
            ])
        elif speed < 25:
            return ("良好", "适合高清视频流媒体和在线游戏", [
                "保持网络设备固件更新",
                "定期检查网络速度",
                "确保没有不必要的网络限制"
            ])
        elif speed < 50:
            return ("优秀", "适合4K视频流媒体和大型文件下载", [
                "确保设备网卡支持高速连接",
                "考虑使用千兆网络设备",
                "定期进行网络优化"
            ])
        elif speed < 100:
            return ("极佳", "适合多设备同时4K流媒体", [
                "检查网络设备是否达到最佳性能",
                "考虑使用企业级网络设备",
                "确保ISP提供的实际速度符合承诺"
            ])
        else:
            return ("卓越", "专业级网络性能", [
                "确保所有网络设备支持千兆速度",
                "考虑使用光纤连接",
                "定期进行网络维护和优化"
            ])

    def get_ping_evaluation(self, ping):
        if ping == "请求超时" or "失败" in ping or "错误" in ping:
            return ("不可用", "网络连接存在问题", [
                "检查目标服务器是否在线",
                "确认本地网络连接正常",
                "尝试更换DNS服务器",
                "检查防火墙设置"
            ])
        
        try:
            ping_value = float(ping.split()[0])
            if ping_value > 300:
                return ("非常差", "网络延迟极高，不适合实时应用", [
                    "检查网络拥塞情况",
                    "尝试连接更近的服务器",
                    "考虑使用有线连接",
                    "关闭占用带宽的应用"
                ])
            elif ping_value > 200:
                return ("较差", "有明显延迟，影响实时体验", [
                    "优化网络路由",
                    "检查ISP连接质量",
                    "考虑使用游戏加速器(如适用)",
                    "避免网络高峰时段使用"
                ])
            elif ping_value > 100:
                return ("一般", "适合普通浏览，游戏体验一般", [
                    "关闭后台更新",
                    "检查路由器QoS设置",
                    "优先使用5GHz WiFi"
                ])
            elif ping_value > 50:
                return ("良好", "适合大多数在线游戏", [
                    "保持网络设备更新",
                    "定期检查网络延迟",
                    "优化设备位置减少干扰"
                ])
            else:
                return ("优秀", "极低延迟，适合竞技游戏", [
                    "保持当前网络设置",
                    "定期进行网络维护",
                    "考虑使用专用游戏网络(如适用)"
                ])
        except:
            return ("未知", "无法评估Ping质量", [
                "检查测试结果格式",
                "重新运行Ping测试",
                "确认网络连接稳定"
            ])

    def get_overall_evaluation(self, speed_eval, ping_eval):
        speed_rank = ["极差", "非常差", "较差", "一般", "良好", "优秀", "极佳", "卓越"].index(speed_eval[0])
        ping_rank = ["不可用", "非常差", "较差", "一般", "良好", "优秀"].index(ping_eval[0] if ping_eval[0] in ["不可用", "非常差", "较差", "一般", "良好", "优秀"] else "一般")
        
        overall_score = (speed_rank * 2 + ping_rank) / 3  # Give download speed more weight
        
        if overall_score < 1.5:
            return ("非常差", "网络质量极不理想", [
                "立即检查网络连接",
                "联系网络服务提供商",
                "考虑更换网络服务",
                "全面检查网络硬件"
            ])
        elif overall_score < 3:
            return ("较差", "网络体验不佳", [
                "优化网络设置",
                "检查设备连接质量",
                "考虑升级网络套餐",
                "避开网络使用高峰时段"
            ])
        elif overall_score < 5:
            return ("一般", "满足基本需求", [
                "定期监测网络速度",
                "优化路由器位置",
                "关闭不必要网络应用",
                "考虑网络升级计划"
            ])
        elif overall_score < 7:
            return ("良好", "适合大多数用途", [
                "保持当前网络设置",
                "定期重启网络设备",
                "监控网络使用情况",
                "考虑进一步优化"
            ])
        else:
            return ("优秀", "高质量网络连接", [
                "保持良好网络习惯",
                "定期维护网络设备",
                "享受高速网络体验",
                "考虑专业级网络优化"
            ])

    def closeEvent(self, event):
        self.cleanup_threads()
        event.accept()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    app.setStyle('Fusion')  # Modern style
    
    # Set application font
    font = QFont('Microsoft YaHei', 10)
    app.setFont(font)
    
    ex = SpeedTestApp()
    ex.show()
    sys.exit(app.exec_())
